
//
//  '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
//  '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
//  '' Name				:3D_Math_Functions 
//  '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
//  '' Purpose			: Support routines for 3D object manipulations 
//  ''				: 
//  '' Includes			: 
//  '' Requirements		: 
//  '' Programmer		: William Tagliaferro, ENS Inc.
//  '' Version			: 1.0
//  '' Date Modified		: 04 December 2001
//  '' Modified By		: William Tagliaferro
//  '' Future Develop't		: 
//  ''                      
//  '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
//  '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
//  ''''''''''''Global Variables'''''''''''''''''''''''''''''''''''''''''

// eval the placeholder clip 
moclip = eval (_root[getProperty (_parent._parent, _name)]);



deg2rad = Math.PI/180.0;	//Convert degrees to radians
att = new Array(0, 0, 0);	//Initialize the object attitude vector
org = new Array(1, 1, 0);	//Define the origin of the mouse
eye = 400;			//Location of the viewer's eye from the screen
moclip.plain = 0;		//Defines the plane of rotation

				//	plain = 0 xy 
				//	plain = 1 yz
				//	plain = 2 xz 
lnum = 0;			//MC level counter
cno = 0;			//MC level counter
moclip.mline._visible = 0;	//Hide the offstage line

//'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
//''''''''''''FUNCTIONS''''''''''''''''''''''''''''''''''''''''''''''''
//'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
//	ObjPoint(x,y,z,xc,yc,MC,opt)
//
//		opt = 0 - left hand cartesian system centered at object center of mass
//		x = object frame x position
//		y = object frame y position
//		z = object frame z position
//		opt = 1 - polar system centered at object center of mass
//		x = radius
//		y = object frame y position
//		z = angle measured clockwise
//		xc = display frame position of object
//		yc = display frame position of object
//		MC = movie clip to represent this point or "none"
//		opt = coordinate system to use (see above)
//

//function ObjPoint (x, y, z, cx, cy, MC, opt) {
function ObjPoint (x, y, z, cx, cy, MC, opt) {
	cno++;
	if(opt == 1) {//x and z in polar form (r,theta)
		r = x;
		theta = z*deg2rad;
		x = r*Math.sin(theta);
		z = r*Math.cos(theta);
	}
	this.p = new Array(x,y,z)	// position vector
	this.pp = new Array(x,y,z)	// perspective position
	this.xloc = cx;			// screen x-position
	this.yloc = cy;			// screen y-position
	this.name = MC+cno;		// path to movie clip for this point
	this.scale = 1.0;

	if(MC != "none" && displaypoints==true) moclip.attachMovie(MC, MC+cno, 30+cno);
	return;


}
function ObjMoveX(dist) {
	this.xloc += dist;
}
function ObjMoveY(dist) {
	this.yloc += dist;
}
function render () {

	obj 	= moclip[this.name];
	obj._x 	= this.pp[0] + this.xloc;
	obj._y 	= this.pp[1] + this.yloc;
	obj._xscale = this.scale*100;
	obj._yscale = this.scale*100;
	obj._alpha 	= this.scale*100;
	return;
}
function xrot (t) {
	// rotates ccw about x axis (right) looking toward the origin
	ang = t*deg2rad;
	x =  this.p[0];
	y =  this.p[1]*Math.cos(ang)  + this.p[2]*Math.sin(ang);
	z = -this.p[1]*Math.sin(ang)  + this.p[2]*Math.cos(ang);
	this.p[0] = x;
	this.p[1] = y;
	this.p[2] = z;
}
function yrot (t) {
	// rotates ccw about y axis (down) looking toward the origin
	ang = t*deg2rad;
	x = this.p[0]*Math.cos(ang) - this.p[2]*Math.sin(ang);
	y = this.p[1];
	z = this.p[0]*Math.sin(ang) + this.p[2]*Math.cos(ang);
	this.p[0] = x;
	this.p[1] = y;
	this.p[2] = z;
}
function zrot (t) {
	// rotates ccw about z axis (out) looking toward the  origin
	ang = t*deg2rad;
	x =  this.p[0]*Math.cos(ang)  + this.p[1]*Math.sin(ang);
	y = -this.p[0]*Math.sin(ang)  + this.p[1]*Math.cos(ang);
	z =  this.p[2];
	this.p[0] = x;
	this.p[1] = y;
	this.p[2] = z;
}
// 
// 
// Att =   Attitude
// [0] - roll (CCW about X)
// [1] - heading (CCW about y)
// [2] - pitch (CCW about Z)
// 
// 
function rotate (att) {
	sinr = Math.sin(att[0]*deg2rad);
	cosr = Math.cos(att[0]*deg2rad);
	sinp = Math.sin(att[1]*deg2rad);
	cosp = Math.cos(att[1]*deg2rad);
	siny = Math.sin(att[2]*deg2rad);
	cosy = Math.cos(att[2]*deg2rad);
	// r for NED
	r00 = cosy*cosp;
	r10 = cosy*sinp*sinr-siny*cosr;
	r20 = cosy*sinp*cosr+siny*sinr;
	r01 = siny*cosp;
	r11 = siny*sinp*sinr+cosy*cosr;
	r21 = siny*sinp*cosr-cosy*sinr;
	r02 = -sinp;
	r12 = cosp*sinr;
	r22 = cosp*cosr;
	x = this.p[0]*r00 + this.p[1]*r01 + this.p[2]*r02;
	y = this.p[0]*r10 + this.p[1]*r11 + this.p[2]*r12;
	z = this.p[0]*r20 + this.p[1]*r21 + this.p[2]*r22;
	this.p[0] = x;
	this.p[1] = y;
	this.p[2] = z;
}


function perspective (eye) {
	f = (this.p[2] + eye)/eye;
	this.pp[0] = this.p[0]*f;
	this.pp[1] = this.p[1]*f;
	this.scale = f;
}
function drawline (p1, p2, clr) {
	lnum++;
	duplicateMovieClip ("moclip.mline", "line" + lnum, lnum);
	obj = this["line"+lnum];
	obj._x = p1.pp[0] + p1.xloc;
	obj._y = p1.pp[1] + p1.yloc;
	xscale = p2.pp[0] - p1.pp[0];
	yscale = p2.pp[1] - p1.pp[1];
	setProperty (obj, _xscale, xscale);
	setProperty (obj, _yscale, yscale);
	if (clr) {
		myColor = new Color(obj);
		if (clr == 1) myColor.setRGB(0x990000);
		if (clr == 2) myColor.setRGB(0x009900);
		if (clr == 3) myColor.setRGB(0x000099);
		if (clr == 4) myColor.setRGB(0x999900);
		if (clr == 5) myColor.setRGB(0x009999);
	}
}

ObjPoint.prototype = ObjPoint.prototype.constructor;
ObjPoint.prototype.ObjMoveX = ObjMoveX;
ObjPoint.prototype.ObjMoveY = ObjMoveY;
ObjPoint.prototype.rotate = rotate;
ObjPoint.prototype.perspective = perspective;
ObjPoint.prototype.render = render;

function getMouse() {
//trace (plain);
	if(plain == 0) {
		att[0] = -(org[1]-_ymouse)/org[1] * 0.03;	//roll
		att[1] =  (org[0]-_xmouse)/org[0] * 0.03;	//heading
		att[2] =  0.0;							//pitch
	} else if(plain == 1) {
		att[0] =   0.0;							//roll
		att[1] =  (org[0]-_xmouse)/org[0] * 0.03;	//heading
		att[2] =  (org[1]-_ymouse)/org[1] * 0.03;	//pitch
	} else if(plain == 2) {
		att[0] = -(org[1]-_ymouse)/org[1] * 0.03;	//roll
		att[1] =  0.0;							//heading
		att[2] =  (org[0]-_xmouse)/org[0] * 0.03;	//pitch
	}
}





